# Hector --- A collection manager.
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import bl3d

InvalidValue = bl3d.InvalidValue
StringTooShort = bl3d.StringTooShort
StringTooLong = bl3d.StringTooLong
NumberTooSmall = bl3d.NumberTooSmall
NumberTooBig = bl3d.NumberTooBig
DateAndTimeWithoutTimezone = bl3d.DateAndTimeWithoutTimezone
DurationNegative = bl3d.DurationNegative


class UnknownBook(InvalidValue):
    def __init__(self, isbn: str) -> None:
        self.isbn = isbn


class NotAnIsbn(InvalidValue):
    def __init__(self, isbn: str) -> None:
        self.isbn = isbn


class MissingAuthor(InvalidValue):
    def __init__(self) -> None:
        super().__init__("Missing author!")


class BeforeCreationOfIsbn(InvalidValue):
    def __init__(self, year: int) -> None:
        super().__init__("Before creation of the ISBN system!")
        self.year = year



# Hector --- A collection manager.
# Copyright © 2023, 2024 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from typing import Any

from flask import Blueprint, flash, request, session, url_for

from bl_hector.infrastructure.flask import services
from bl_hector.infrastructure.flask.utils import presenter_to_response
from bl_hector.interfaces.to_http import Redirection

blueprint = Blueprint("ip", __name__)


@blueprint.get("/login")
@presenter_to_response
def login() -> Any:
    _ = services.get_translator()
    if not authorized():
        flash(_("ip-login-error"), "error")
        return Redirection(url_for("auth.login"))

    session["user_id"] = "anonymous"

    flash(_("ip-login-success"), "success")
    return Redirection(url_for("auth.redirect"))


def authorized() -> bool:
    authorized_ip = services.get_settings().AUTHORIZED_IP
    client_ip = request.headers.get("X-Remote-IP", request.remote_addr) or ""
    return client_ip.startswith(authorized_ip.rstrip("*"))



# Hector --- A collection manager.
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import datetime

from bl_hector.domain.administration.enumerations import Permissions as P
from bl_hector.domain.administration.repositories import Permissions
from bl_hector.domain.administration.value_objects import UserId
from bl_hector.domain.collection_management.services import Calendar
from bl_hector.domain.collection_management.value_objects import Date

ONLY_USER = "admin"


class DummyPermissions(Permissions):
    def for_user(self, user_id: UserId) -> list[P]:
        return [p for p in P]

    def is_authorized_to(self, user_id: UserId, permission: P) -> bool:
        return bool(str(user_id))


class SystemCalendar(Calendar):
    def today(self) -> Date:
        return Date.instanciate(datetime.date.today())
